/*
  Lights and Texture Example

    Usage
    =====
    q           -- quit
    x/y/z       -- changes axis of rotation
    left mouse  -- rotates entire scene
	t			-- toggles texture on/off

*/

#define APP_NAME                "Lights"
#define X_AXIS                  0
#define Y_AXIS                  1
#define Z_AXIS                  2

#include <stdio.h>
#include <stdlib.h>
#include "tga.h"
#include "glut.h"

/* vector */
typedef float vec3_t[3]; 

/* enumerations for the mouse buttons */
enum {
    UP = 1, 
    DOWN, 
};

/* window width and height */
int winW = 640;
int winH = 480;

/* old position of the mouse */
int oldX = -13;
int oldY = -13;

/* mouse state, UP or DOWN */
int mState = UP;

/* current axis of rotation */
int axisRot = X_AXIS;

/* amount to rotate about axis */
float rotate = 0.0f;

/* vector which describes the axis to rotate about */
vec3_t axis = {1.0, 0.0, 0.0};

/* global rotation, for use with the mouse */
vec3_t gRot = {0,0,0};

/* should we have texture ? */
int enableTexture= 1;

/*
=============
drawAxis
=============

Draws out the X / Y / Z axis just for reference.  
*/
void drawAxis (void)
{
    glColor3f (0.5, 0.5, 0.5);
    glBegin (GL_LINES);
        glColor3f (0.5, 0.0, 0.0);
        glVertex3f (-20.0, 0.0, 0.0);
        glVertex3f (20.0, 0.0, 0.0);

        glColor3f (0.0, 0.5, 0.0);
        glVertex3f (0.0, 20.0, 0.0);
        glVertex3f (0.0, -20.0, 0.0);

        glColor3f (0.0, 0.0, 0.5);
        glVertex3f (0.0, 0.0, -20.0);
        glVertex3f (0.0, 0.0, 20.0);
    glEnd ();
}

// ==============================================
// drawScene
// ==============================================
// Draws the green terrain.  

void drawScene (void)
{
	if (enableTexture)
	{
		glEnable (GL_TEXTURE_2D); /* enable texture mapping */
		glBindTexture (GL_TEXTURE_2D, 1); /* bind to our texture, has id of 1 */
	}
	
	// green terrain
	
	GLfloat materialDiffuse[] = {0.0f, 0.8f, 0.2f, 1.0f};
    GLfloat materialAmbient[] = {0.0f, 0.8f, 0.2f, 1.0f};


	
	glBegin(GL_POLYGON);

		glNormal3f(0.0F, 1.0F, 0.0F);

		//set material
		glMaterialfv(GL_FRONT, GL_DIFFUSE, materialDiffuse);
		glMaterialfv(GL_FRONT, GL_AMBIENT, materialAmbient);

		glTexCoord2f( 0.0F, 1.0F);	
		glVertex3f(-100,0,100);

		glTexCoord2f( 1.0F, 1.0F);	
		glVertex3f(-100,0,-100);

		glTexCoord2f( 0.0F, 0.0F);	
		glVertex3f(100,0,-100);

		glTexCoord2f( 1.0F, 0.0F);	
		glVertex3f(100,0,100);
	glEnd();	

	glDisable (GL_TEXTURE_2D); /* disable texture mapping */

	
}

void drawHouse()
{

	// grey house
	GLfloat materialDiffuse[] = { 0.3, 0.3, 0.3, 1.0 };
	GLfloat materialAmbient[] = { 0.3, 0.3, 0.3, 1.0 };

	glMaterialfv(GL_FRONT, GL_DIFFUSE, materialDiffuse);
	glMaterialfv(GL_FRONT, GL_AMBIENT, materialAmbient);

	glPushMatrix();
		glTranslatef(0, 10/2, 0);// bring to level with scene
		glutSolidCube(10.0);
	glPopMatrix();
}



/*
=============
glutDisplay
=============

The display function. 
*/
void glutDisplay (void)
{
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();

    /* move our eye position 50 units away */
    glTranslatef (0, 0, -50);	  
    /* apply the rotation from the mouse */
    glRotatef (gRot[0], 1.0, 0.0, 0.0);
    glRotatef (gRot[1], 0.0, 1.0, 0.0);

    drawAxis();
    drawScene();
	drawHouse();

    glutSwapBuffers();
}



/* 
=============
glutResize
=============

Resize function, called on app start and on resize.
*/
void glutResize (int w, int h)
{	
    if (!h)
       return;

    winW = w;
    winH = h;

    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glViewport (0, 0, winW, winH);

    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();

    gluPerspective (90, winW / winH, 1, 9999);

    glutPostRedisplay ();
}
/* 
=============
glutKeyboard
=============

Keyboard handler.
*/
void glutKeyboard (unsigned char key, int x, int y)
{
    switch (key)
    {
        case 'q':
        case 'Q':
            exit (1);
        break;
        case 'x':
        case 'X':
            /* axis of rotation is now X */
            axisRot = X_AXIS;
            axis[0] = 1.0;
            axis[1] = axis[2] = 0.0;
        break;
        case 'y':
        case 'Y':
            /* axis of rotation is now Y */
            axisRot = Y_AXIS;
            axis[1] = 1.0;
            axis[0] = axis[2] = 0.0;
        break;
        case 'z':
        case 'Z':
            /* axis of rotation is now Z */
            axisRot = Z_AXIS;
            axis[2] = 1.0;
            axis[1] = axis[0] = 0.0;
        break;
        case 't':
		case 'T':
            enableTexture = !enableTexture;
        break;
    }
    glutPostRedisplay ();
}
/*
=============
clamp
=============

Clamps the rotation to -360 to 360
*/
void clamp (vec3_t v)
{
    int i;

    for (i = 0; i < 3; i ++)
        if (v[i] > 360 || v[i] < -360)
            v[i] = 0.0f;
}
/*
=============
glutMotion
=============

Called when the mouse moves in our app area. 
*/
void glutMotion(int x, int y) 
{
    if (mState == DOWN) 
    {
        gRot[0] -= ((oldY - y) * 180.0f) / 100.0f;
        gRot[1] -= ((oldX - x) * 180.0f) / 100.0f;
        clamp (gRot);
        glutPostRedisplay ();
    } 
    oldX = x; 
    oldY = y;
}
/*
=============
glutMouse
=============

Called when the mouse is clicked.
*/
void glutMouse(int button, int state, int x, int y) 
{
    if(state == GLUT_DOWN) 
    {
        switch(button) 
        {
            case GLUT_LEFT_BUTTON:
            case GLUT_RIGHT_BUTTON:
                mState = DOWN;
                oldX = x;
                oldY = y;
            break;
        }
    } else if (state == GLUT_UP) 
        mState = UP;
}
/* 
=============
glInit
=============

Sets up various OpenGL stuff.
*/
void glInit (void)
{
    glEnable (GL_DEPTH_TEST);
	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
	glClearDepth(1.0);
	glClearColor(0, 0.1, 0.9, 1);

	
	/* lights on! */

	GLfloat light_ambient[] = { 0.3, 0.3, 0.3, 1.0 };
    GLfloat light_diffuse[] = { 0.3, 0.3, 0.3, 1.0 };
    GLfloat light_specular[] = { 0.3, 0.3, 0.3, 1.0 };

    GLfloat light_position[] = { 4.0, 40.0, 2.0, 20.0 };

    glLightfv (GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv (GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv (GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv (GL_LIGHT0, GL_POSITION, light_position);
   
    glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0);

		
	// load textures

	Tga image;
	    	
	image.Load("green.tga", 1); // assign to number

}



void main (void)
{
    glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize (winW,winH);
    glutCreateWindow (APP_NAME);
    glutKeyboardFunc (glutKeyboard);
    glutDisplayFunc (glutDisplay);
    glutReshapeFunc (glutResize);
    glutMotionFunc (glutMotion);
    glutMouseFunc (glutMouse);
    
    
    glInit ();
    glutMainLoop();
}